import {BryntumGridProps, BryntumResourceHistogramProps, BryntumSchedulerProProps} from '@bryntum/schedulerpro-react'
import {Column, DateHelper, ProjectModel, ResourceModel, ResourceModelConfig, WidgetHelper} from '@bryntum/schedulerpro'

const dailyPreset = {
    base: 'hourAndDay',
    tickWidth: 45,
    timeResolution: {              // Dates will be snapped to this resolution
        unit: 'minute',       // Valid values are 'millisecond', 'second', 'minute', 'hour', 'day', 'week', 'month', 'quarter', 'year'.
        increment: 1
    },
    headers: [
        {
            unit: 'day',
            align: 'center',
            dateFormat: 'dddd DD.MM'
        },
        {
            unit: 'h',
            align: 'center',
            dateFormat: 'HH'
        }
    ]
}

const weeklyPreset = {
    base: 'dayAndWeek',
    // id: 'myPreset',              // Unique id value provided to recognize your view preset. Not required, but having it you can simply set new view preset by id: scheduler.viewPreset = 'myPreset'

    // name: 'My view preset',        // A human-readable name provided to be used in GUI, e.i. preset picker, etc.

    // tickWidth  : 24,                // Time column width in horizontal mode
    // tickHeight : 50,                // Time column height in vertical mode
    displayDateFormat: 'DD HH:mm',    // Controls how dates will be displayed in tooltips etc

    shiftIncrement: 1,             // Controls how much time to skip when calling shiftNext and shiftPrevious.
    shiftUnit: 'day',         // Valid values are 'millisecond', 'second', 'minute', 'hour', 'day', 'week', 'month', 'quarter', 'year'.
    //defaultSpan: 7,            // By default, if no end date is supplied to a view it will show 12 hours

    timeResolution: {              // Dates will be snapped to this resolution
        unit: 'minute',       // Valid values are 'millisecond', 'second', 'minute', 'hour', 'day', 'week', 'month', 'quarter', 'year'.
        increment: 10
    },

    headers: [                     // This defines your header rows from top to bottom
        {                           // For each row you can define 'unit', 'increment', 'dateFormat', 'renderer', 'align', and 'thisObj'
            unit: 'month',
            increment: 1,
            dateFormat: 'MMMM'
        },
        {
            unit: 'day',
            increment: 1,
            dateFormat: 'ddd DD.MM'
        },
    ],

    // columnLinesFor : 1              // Defines header level column lines will be drawn for. Defaults to the last level.
}

const monthlyPreset = {
    id: 'monthlyView',
    tickWidth: 35,
    //rowHeight         : 32,
    displayDateFormat: 'DD.MM',
    shiftIncrement: 1,
    shiftUnit: 'day',
    timeResolution: {
        unit: 'day',
        increment: 1
    },
    //defaultSpan: 31,
    mainHeaderLevel: 1,
    headers: [
        {
            unit: 'month',
            increment: 1,
            dateFormat: 'MMMM YYYY'
        },
        {
            unit: 'day',
            increment: 1,
            dateFormat: 'DD'
        }
    ]
}

// Upper-level SchedulerPro getter
const getScheduler = (child: any) => {
    return child.up('schedulerpro')
}

class MyResource extends ResourceModel {
    //https://forum.bryntum.com/viewtopic.php?t=18441&start=10
    constructor(config?: Partial<ResourceModelConfig>) {
        super(config)
        Object.defineProperty(this, 'initials', {
            get() {
                return 'ABC'
            }
        })
    }
}

const project = new ProjectModel({
    resourceModelClass: MyResource,
})


const schedulerProConfig: BryntumSchedulerProProps = {
    viewPreset: weeklyPreset,
    allowOverlap: false,
    rowHeight: 80,
    flex: '1 1 60%',
    enableUndoRedoKeys: true,
    stripeFeature: true,
    timeRangesFeature: true,
    resourceMargin: 25,
    columns: [
        {
            type: 'resourceInfo',
            text: 'Name',
            field: 'name',
            showEventCount: true,
            width: 200,
            editor: null,
        }
    ],
    project: project,

    calendarHighlightFeature: true,
    eventTooltipFeature: {
        hoverDelay: 300,
        textContent: true,
        template: (event: any) => {
            return `<h4>ID: ${event.eventRecord.id}</h4>`
        },
    },
    features: {
        tree: true,
        //To customize arrow between mwos on timeline
        //https://www.bryntum.com/docs/gantt/api/Scheduler/feature/Dependencies
        dependencies: true,
        //https://www.bryntum.com/docs/gantt/api/Scheduler/feature/DependencyEdit
        dependencyEdit: false,
        eventDrag: {
            constrainDragToResource: true,
        },
        eventDragCreate: false,
        eventDragSelect: true,
        eventResize: false,
        cellMenu: true,

        timeAxisHeaderMenu: {
            disabled: true,
        },
        //For right click on events
        //https://www.bryntum.com/docs/gantt/api/Scheduler/feature/EventMenu
        eventMenu: {
            disabled: true
        },
        //Right click on empty space in timeline
        scheduleMenu: {
            disabled: true
        },


        nonWorkingTime: true,
        resourceNonWorkingTime: {
            maxTimeAxisUnit: 'week',
        },
    },


    extraData: {
        startHour: 0,
        endHour: 24
    },

    zoomKeepsOriginalTimespan: true,
    zoomOnTimeAxisDoubleClick: false,
    zoomOnMouseWheel: false,
    createEventOnDblClick: false,
    enableDeleteKey: false,
    enableEventAnimations: false,
    multiEventSelect: false,
    highlightPredecessors: true,
    highlightSuccessors: true,
    mode: 'horizontal'
}

const histogramConfig: BryntumResourceHistogramProps = {
    project: project,
    hideHeaders: true,
    rowHeight: 60,
    flex: '1 1 40%',
    showBarTip: true,
    showBarText: false,
    columns: [
        {
            type: 'resourceInfo',
            text: 'Name',
            field: 'name',
            editor: false,
            //enableCellContextMenu: false,
            showEventCount: true,
            width: 200
        },
        {
            type: 'scale',
            hidden: true,
        }
    ],

    features: {
        nonWorkingTime: true,
        resourceNonWorkingTime: {
            maxTimeAxisUnit: 'week',
        },
    },

    calendarHighlightFeature: true,

    eventTooltipFeature: {
        hoverDelay: 300,
        textContent: true,
        template: (event: any) => {
            return `<h4>ID: ${event.eventRecord.id}</h4>`
        },
    },
}

const gridConfig: BryntumGridProps = {
    flex: 1,
    rowHeight: 60,
    readOnly: false,
    minWidth: 250,
    maxWidth: 500,
    selectionMode: {
        row: true
    },
    draggable: true,

    stripeFeature: true,
    sortFeature: 'id',
    cellTooltipFeature: {
        hoverDelay: 300,
        textContent: true,
        tooltipRenderer: ({record: order}: any) =>
            order.isScheduled ?
                `<h4>Type: ${order.templateName}</h4>This order is scheduled to finish by <b>${DateHelper.format(order.finishDate, 'MMM d HH:mm')}</b>`
                : `<h4>Type: ${order.templateName}</h4>Try dragging this order onto the schedule`
    },

    // Grid top toolbar

    columns: [{
        type: 'template',
        text: 'Order List',
        flex: 1,
        maxWidth: 100,
        field: 'name',
        editor: false,
        htmlEncode: false,
        cellCls: 'order-cell',
        template: ({record: order}: any) =>
            `<div>${'#' + order.id} </div>
            <div class="customer">Descr: ${order.name}</div>`
    }, {
        text: 'Start',
        type: 'date',
        flex: 1,
        align: 'right',
        format: 'MMM D HH:mm',
        field: 'startDate',
    }, {
        text: 'Finish',
        type: 'date',
        flex: 1,
        align: 'right',
        format: 'MMM D HH:mm',
        editor: false,
        field: 'endDate'
    },
    {
        text: 'Due date',
        type: 'date',
        flex: 1,
        align: 'right',
        format: 'MMM D HH:mm',
        editor: false,
        field: 'dueDate'
    },

    ],

    disableGridRowModelWarning: true
}

export {
    schedulerProConfig,
    gridConfig,
    dailyPreset,
    weeklyPreset,
    monthlyPreset,
    getScheduler,
    histogramConfig,
    project
}
