import React, {Fragment, useCallback, useEffect, useRef, useState} from 'react'
import {
    dailyPreset,
    getScheduler,
    gridConfig,
    histogramConfig,
    monthlyPreset, project,
    schedulerProConfig,
    weeklyPreset
} from './config'
import {BryntumGrid, BryntumResourceHistogram, BryntumSchedulerPro, BryntumSplitter} from '@bryntum/schedulerpro-react'
import {
    CalendarModel,
    DateHelper,
    ResourceHistogram,
    SchedulerPro,
    TimeRangesConfig,
    Grid,
    Store,
    EventStore,
    EventModel,
    Model,
    DomClassList,
    ScrollManager,
    WidgetHelper,
    ResourceModel, SchedulerEventModel, TimeSpan
} from '@bryntum/schedulerpro'
import './Timeline.scss'
import {Drag} from './Drag'
import {
    Button,
    Checkbox,
    Dialog,
    DialogActions,
    DialogContent,
    DialogTitle,
    FormControlLabel,
    TextField,
    Grid as MuiGrid,
} from '@mui/material'
import {autoPlanStateType, CoreType, setPlanStateType} from "../../App";
import {MwoType} from "../../models/mwo";
import {FwoType} from "../../models/fwo";
import {mwoId} from "../../models/id";
import {ErrorBaseEnum, ErrorType} from "../../models/error";

type TimelineProps = CoreType & {
    setMwos: React.Dispatch<React.SetStateAction<MwoType[]>>,
    setFwos: React.Dispatch<React.SetStateAction<FwoType[]>>,
    planState: autoPlanStateType,
    setPlanState: setPlanStateType,
}

type IdEvent = string
type IdResource = string
type IdDependency = string
type IdAssignment = string
type IdCalendar = string

enum DependencyTypeType {
    startToStart,
    startToEnd,
    endToStart,
    endToEnd,
}

type DependencyType = {
    id: IdDependency,
    from: mwoId,
    to: mwoId,
    lag: number,
    lagUnit: string
    cls: string
    type: DependencyTypeType
}

type ResourceType = {
    id: IdResource
    name: string,
    //iconCls: string,
    calendar: CalendarModel,
}
type AssignmentType = {
    id: IdAssignment,
    resourceId: IdResource,
    eventId: IdEvent
}

type EventType = {
    id: IdEvent
    name: string
    startDate: Date | null
    endDate: Date | null
    iconCls: string
    //eventColor: string
    resizable: boolean
    cls: string | null
    //dueDate: Date
    resourceId: string
}

type GridEventType = {
    id: IdEvent
    name: string
    startDate: Date | null
    endDate: Date | null
    dueDate: Date | null
}

type intervalType = {
    name: string
    cls?: string
    endDate?: Date
    iconCls?: string
    isWorking: boolean
    recurrentEndDate?: string
    recurrentStartDate?: string
    startDate?: Date
}

type CalendarType = {
    id: IdCalendar
    name: string
    unspecifiedTimeIsWorking: boolean
    cls?: string
    intervals: intervalType[]
}

//SchedulerEventModel
const extractEvents = (data: CoreType): EventType[] => {
    //console.log('extractEvents: ' + data.mwos.length)
    return data.mwos.map(mwo => {
        let ed: Date | null
        let iconCls = ''

        //const fwoIndex = data.fwos.findIndex(value => value.id === mwo.fwoId)


        if (mwo.startPlanned !== null && mwo.endDate !== null) {
            ed = new Date(mwo.startPlanned?.getTime())
            //const ed2 = DateHelper.add(ed, mwo.duration, 'seconds')
            ed.setSeconds(ed.getSeconds() + mwo.duration)
            mwo.endDate = ed
        }

        if (mwo.startPlannedLocked) {
            iconCls = 'b-fa b-fa-lock'
        }

        return {
            id: mwo.id,
            name: mwo.description,
            startDate: mwo.startPlanned,
            endDate: mwo.endDate,
            image: false,
            resourceId: mwo.wksId,
            iconCls: iconCls,
            duration: mwo.duration,
            durationUnit: 'second',
            resizable: false,
            cls: null
        }
    })
}

const extractData = (data: CoreType): EventType[] => {
    return data.mwos.map(mwo => {
        let ed: Date | null

        if (mwo.startPlanned !== null && mwo.endDate !== null) {
            ed = new Date(mwo.startPlanned?.getTime())
            //const ed2 = DateHelper.add(ed, mwo.duration, 'seconds')
            ed.setSeconds(ed.getSeconds() + mwo.duration)
            mwo.endDate = ed
        }

        return {
            id: mwo.id,
            name: mwo.description,
            startDate: mwo.startPlanned,
            endDate: mwo.endDate,
            image: false,
            resourceId: mwo.wksId,
            iconCls: 'b-fa b-fa-snowplow',
            eventColor: 'orange',
            duration: mwo.duration,
            durationUnit: 'second',
            resizable: false,
            dueDate: mwo.dueDate,
            cls: null,
        }
    })
}


const extractResources = (data: CoreType): ResourceType[] => {
    return data.wks.map(wks => {
        return {
            id: wks.id,
            name: wks.description,
            calendar: wks.calendar,
            image: false,
            //iconCls: 'b-fa b-fa-snowplow',
        }
    })
}

/*const extractAssignments = (data: CoreType): AssignmentType[] => {
    let id = 0
    const assignments: AssignmentType[] = []
    for (const mwo of data.mwos) {
        if (mwo.wksId !== null && mwo.startPlanned !== null) {
            id++
            assignments.push({
                id: 'A' + id.toString(),
                resourceId: mwo.wksId,
                eventId: mwo.id
            })
        }
    }
    return assignments
}*/

const extractDependencies = (data: CoreType): DependencyType[] => {
    //let id = 0
    const dependencies: DependencyType[] = []

    data.mwos.forEach((value, index) => {
        if (value.predecessors !== null) {
            dependencies.push({
                id: 'D' + index.toString(),
                from: value.predecessors,
                to: value.id,
                lag: 0,
                lagUnit: 's',
                cls: 'customDependency',
                type: DependencyTypeType.endToStart
            })
        }
    })

    return dependencies
}

const extractCalendars = (data: CoreType): CalendarType[] => {
    return data.calendars
}

const timeRangesSchedule: Partial<TimeRangesConfig> = {
    // This displays the red line indicating the current time
    showCurrentTimeLine: true,
    //currentDateFormat: 'D.M',
}

const timeRangesHistogram: Partial<TimeRangesConfig> = {
    // For histogram we don't want the time so we just insert empty format.
    showCurrentTimeLine: true,
    currentDateFormat: '',
}


const visibleDate = {
    date: new Date(),
    block: 'center'
}

enum View {
    daily,
    weekly,
    monthly,
}

const ViewLength = {
    [View.daily]: 1,
    [View.weekly]: 7,
    [View.monthly]: 31
}


const Timeline = (props: TimelineProps) => {
    const [filterDialog, setFilterDialog] = useState(false)
    const [createBookMarkDialog, setCreateBookMarkDialog] = useState(false)
    const [hideHistogram, setHideHistogram] = useState(false)
    const [editPanel, setEditPanel] = useState(false)
    const [editPanelData, setEditPanelData] = useState<SchedulerEventModel | null>(null)

    const histogramRef = useRef<BryntumResourceHistogram>(null)
    const schedulerRef = useRef<BryntumSchedulerPro>(null)
    const gridRef = useRef<BryntumGrid>(null)

    const histogramInstance = () => histogramRef?.current?.instance as ResourceHistogram
    const gridInstance = () => gridRef?.current?.instance as Grid
    const schedulerInstance = () => schedulerRef?.current?.instance as SchedulerPro


    // Toggle MWO list visibility
    const onToggleOrderList = useCallback(() => {
        if (gridRef !== null && gridRef.current !== null) {
            gridInstance().hidden = !gridInstance().hidden
            const element = document.getElementById('schedule_container')!
            if (!gridInstance().hidden) {
                element.style.width = '80%'
            }
        }

        bryntumIconsClassGrid()
    }, [])

    useEffect(() => {
        histogramInstance().addPartner(schedulerInstance())
    }, [schedulerRef, schedulerRef.current, gridRef, gridRef.current, histogramRef, histogramRef.current])


    useEffect(() => {
        const element = document.getElementById('schedule_container')!
        element.style.width = '80%'

    }, [])

    const addDraggedElement = useCallback((obj: SchedulerEventModel) => {
        try {
            const index = props.mwos.findIndex(el => el.id === obj.id)

            props.mwos[index].startPlanned = obj.startDate as Date

            const ed = new Date((obj.startDate as Date).getTime())
            ed.setSeconds(ed.getSeconds() + props.mwos[index].duration)
            props.mwos[index].endDate = ed
            obj.endDate = ed
            obj.name = props.mwos[index].description
            obj.resourceId = props.mwos[index].wksId
            obj.duration = props.mwos[index].duration

            const event = schedulerInstance().eventStore.add({
                ...obj,
            })

            const store: Store = gridInstance().store as Store
            //If there is a filter to grid (filter for hiding scheduled items). we remove it
            if (store.isFiltered) {
                store.remove(obj.id)
            } else {
                store.add(
                    {
                        ...obj,
                        dueDate: props.mwos[index].dueDate
                    }
                )
            }
        } catch (e) {
            console.log(e)
        }
    }, [props.mwos, props.fwos])

    const onEventSelectionChange = useCallback(({selected}: { selected: SchedulerEventModel[] }) => {
        if (selected.length > 0) {
            setEditPanelData(selected[0])
        }
    }, [])


    useEffect(() => {
        // // Setup dragging
        new Drag({
            grid: gridRef?.current?.instance,
            schedule: schedulerRef?.current?.instance,
            constrain: false,
            outerElement: gridRef?.current?.instance.element,
            scheduleInstance: schedulerInstance(),
            histogramInstance: histogramInstance(),
            gridInstance: gridInstance(),
            addDraggedElement: addDraggedElement
        })
    }, [])

    // Runs only once. Dependencies are not going to change
    useEffect(() => {
        const startDate = new Date()
        const endDate = new Date()

        startDate.setMonth(startDate.getMonth() - 6)
        endDate.setMonth(endDate.getMonth() + 6)

        if (schedulerRef !== null && schedulerRef?.current !== null) {
            const scheduler = schedulerRef.current.instance
            scheduler.on({
                toggleOrderList: onToggleOrderList,
            })

            schedulerInstance().setTimeSpan(startDate, endDate)
            schedulerInstance().scrollToNow({block: 'center'})
        }
    }, [])

    const onDataChange = (event: any): void => {
        if (event.action === 'update') {
            const index = props.mwos.findIndex(el => el.id === event.record.originalData.id)
            if (event.changes.startDate !== undefined) {

                props.mwos[index].startPlanned = event.changes.startDate.value
                let ed = event.changes.startDate.value
                if (ed !== null && ed !== undefined) {
                    ed = new Date(ed.getSeconds())
                    ed.setSeconds(props.mwos[index].duration)
                    props.mwos[index].endDate = ed
                }

                if (event.changes.startDate.value > props.mwos[index].dueDate) {
                    if (!props.mwos[index].errors.some((el: ErrorType) => el.base === ErrorBaseEnum.overDue)) {
                        props.mwos[index].errors.push(
                            {
                                description: 'Over Due',
                                base: ErrorBaseEnum.overDue,
                            }
                        )
                    }

                } else {
                    const baseIndex = props.mwos[index].errors.findIndex(el => el.base === ErrorBaseEnum.overDue)
                    props.mwos[index].errors.splice(baseIndex, 1)

                }
            }
            if (event.changes.duration !== undefined) {
                props.mwos[index].duration = event.changes.duration.value
                event.record.duration = event.changes.duration.value
            }
            if (event.changes.endDate !== undefined) {
                event.record.endDate = event.changes.endDate.value
            }
            if (event.changes.draggable !== undefined) {
                event.record.draggable = event.changes.draggable
            }
        }
    }

    const setNewActivePreset = (event: any): void => {
        const elements = document.getElementsByClassName('bryntumButtonGroup')
        Array.from(elements).forEach(el => el.classList.remove('bryntumButtonGroupActive'))

        event.currentTarget.classList.add('bryntumButtonGroupActive')
    }

    const clearSearch = (): void => {
        const gridStore = gridInstance().store as Store
        const schedulerStore = schedulerInstance().eventStore as EventStore

        gridStore.clearFilters()
        schedulerStore.forEach((task: SchedulerEventModel) => {
            const taskClassList = new DomClassList(task.cls)

            taskClassList.remove('searchedWidget')

            task.cls = taskClassList.value
        })
    }

    const searchInputChange = (value: string): void => {
        if (value === undefined || value === '') {
            clearSearch()
            return
        }

        const gridStore: Store = gridInstance().store as Store
        const schedulerStore: EventStore = schedulerInstance().eventStore


        value = value.toLowerCase()
        //Search for grid
        gridStore.clearFilters()
        gridStore.filter((el: Model) => {
            return el.id.toString().toLowerCase().includes(value) || el.getData('name').toLowerCase().includes(value)
        })

        //Search for scheduler
        schedulerStore.forEach((task: SchedulerEventModel) => {
            const taskClassList = new DomClassList(task.cls)

            taskClassList.remove('searchedWidget')

            if ((task.id.toString().toLowerCase().includes(value) || task.name.toLowerCase().includes(value))) {
                taskClassList.add('searchedWidget')
                schedulerInstance().scrollToDate(task.startDate as Date, {block: 'center'})
            }

            task.cls = taskClassList.value
        })
    }

    // Don't allow dragging events that are locked
    const beforeEventDrag = ({eventRecord}: any): boolean => {
        return !props.mwos.find(value => value.id === eventRecord.id)?.startPlannedLocked
    }

    const bryntumIconsClass = (condition: boolean): string => {
        return condition ? 'bryntumIconButtonBlue' : 'bryntumIconButtonGray'
    }

    const bryntumIconsClassGrid = (): void => {
        const element = document.getElementsByClassName('bGridToggle')
        element[0].classList.remove('bryntumIconButtonGray')
        element[0].classList.remove('bryntumIconButtonBlue')


        if (gridRef === null || gridRef.current === null) {
            return
        }

        element[0].classList.add(!gridInstance().hidden ? 'bryntumIconButtonBlue' : 'bryntumIconButtonGray')

    }

    // @ts-ignore
    return (
        <div id={'scheduler_container'}
             style={{flex: 'auto', flexWrap: 'nowrap', display: 'flex', flexDirection: 'row'}} className={'scheduler'}>
            <Dialog open={filterDialog}>
                <DialogTitle>
                    Workstations filter
                </DialogTitle>
                <DialogContent>
                    <MuiGrid container justifyContent={'flex-start'} alignItems={'center'}>
                        {
                            props.mwos.map(mwo => (
                                <MuiGrid key={mwo.id}>
                                    <FormControlLabel value={mwo.id} control={<Checkbox checked={true}></Checkbox>}
                                                      label={mwo.id}></FormControlLabel>
                                </MuiGrid>
                            ))
                        }
                    </MuiGrid>
                </DialogContent>
                <DialogActions>
                    <Button id={'filterDialogSave'} onClick={() => {
                        setFilterDialog(!filterDialog)
                    }}>
                        Save
                    </Button>
                    <Button id={'filterDialogCreateBookmark'} onClick={() => {
                        //setFilterDialog(!filterDialog)
                        setCreateBookMarkDialog(!createBookMarkDialog)
                    }}>
                        Create bookmark
                    </Button>
                    <Button id={'filterDialogCancel'} onClick={() => {
                        setFilterDialog(!filterDialog)
                    }}>
                        Cancel
                    </Button>
                </DialogActions>
            </Dialog>
            <Dialog open={createBookMarkDialog}>
                <DialogContent>
                    <TextField id={'bookMarkName'} label={'New bookmark name'} variant={'filled'}></TextField>
                </DialogContent>
                <DialogActions>
                    <Button id={'filterDialogSave'} onClick={() => {
                        setFilterDialog(!filterDialog)
                        setCreateBookMarkDialog(!createBookMarkDialog)
                    }}>
                        Create
                    </Button>
                    <Button id={'filterDialogCancel'} onClick={() => {
                        setCreateBookMarkDialog(!createBookMarkDialog)
                    }}>
                        Cancel
                    </Button>
                </DialogActions>
            </Dialog>
            <Fragment>
                <div id={'schedule_container'}>
                    <BryntumSchedulerPro
                        {...schedulerProConfig}
                        tbar={[
                            {
                                icon: 'b-icon b-fa-caret-left',
                                cls: {'bryntumIconButtonLeft': true},
                                onAction: ({source}: any) => {
                                    schedulerInstance().shiftPrevious()
                                }
                            },
                            {
                                icon: 'b-icon b-fa-caret-right',
                                cls: {'bryntumIconButtonRight': true},
                                onAction: ({source}: any) => {
                                    schedulerInstance().shiftNext()
                                }
                            },
                            {
                                text: 'Now',
                                cls: {'bryntumButton': true},
                                onAction: ({source}: any) => {
                                    schedulerInstance().scrollToNow({block: 'center'})
                                }
                            },
                            {
                                type: 'buttongroup',
                                toggleGroup: true,
                                cls: {'bryntumButtonGroup': true},
                                items: [
                                    {
                                        //id: 'buttonGroupDay',
                                        text: 'Day',
                                        value: 1,
                                        cls: 'bryntumButtonGroup bryntumButtonGroupDay',
                                        preset: dailyPreset,
                                    },

                                    {
                                        //id: 'buttonGroupDayWeek',
                                        text: 'Week',
                                        cls: 'bryntumButtonGroup bryntumButtonGroupWeek bryntumButtonGroupActive',
                                        value: 7,
                                        preset: weeklyPreset,
                                    },
                                    {
                                        //id: 'buttonGroupMonth',
                                        text: 'Month',
                                        cls: 'bryntumButtonGroup bryntumButtonGroupMonth',
                                        value: 31,
                                        preset: monthlyPreset,
                                    },
                                ],
                                onClick: ({source, event}: any) => {
                                    setNewActivePreset(event)
                                    const
                                        scheduler = getScheduler(source),
                                        value = source.value,
                                        startDate = DateHelper.add(DateHelper.clearTime(scheduler.startDate), scheduler.extraData.startHour, 'h'),
                                        endDate = DateHelper.add(startDate, value - 1, 'd')

                                    endDate.setHours(scheduler.extraData.endHour)

                                    schedulerInstance().viewPreset = source.preset
                                }
                            },
                            '->',
                            {
                                type: 'textfield',
                                cls: 'searchText',
                                clearable: true,
                                placeholder: 'Search mwos..',
                                maxLength: 150,
                                triggers: {
                                    plug: {
                                        cls: 'b-fa b-fa-search textFieldIcon'
                                    }
                                },
                                onClear: ({source}: any) => {
                                    clearSearch()
                                },
                                onInput: ({value}: any) => {
                                    searchInputChange(value)
                                }
                            },
                            {
                                icon: 'b-icon b-icon-filter',
                                cls: bryntumIconsClass(filterDialog),
                                onAction: ({source}: any) => {
                                    setFilterDialog(!filterDialog)
                                }
                            },
                            {
                                icon: 'b-icon b-fa-chart-column',
                                cls: bryntumIconsClass(!hideHistogram),
                                onAction: ({source}: any) => {
                                    setHideHistogram(!hideHistogram)
                                }
                            },
                            {
                                icon: 'b-icon b-icon-menu-vertical',
                                cls: 'bGridToggle bryntumIconButtonBlue',
                                onAction: ({source}: any) => {
                                    //getScheduler(source).trigger('toggleOrderList')
                                    schedulerInstance().trigger('toggleOrderList')
                                }
                            }
                        ]}
                        ref={schedulerRef}
                        events={extractEvents(props)}
                        resources={extractResources(props)}
                        //assignments={extractAssignments(props)}
                        dependencies={extractDependencies(props)}
                        calendars={extractCalendars(props)}
                        timeRangesFeature={timeRangesSchedule}
                        visibleDate={visibleDate}
                        scrollable={true}
                        onDataChange={onDataChange}
                        onBeforeEventDrag={beforeEventDrag}
                        onEventSelectionChange={onEventSelectionChange}
                    />
                    <BryntumSplitter/>
                    <BryntumResourceHistogram
                        {...histogramConfig}
                        hidden={hideHistogram}
                        ref={histogramRef}
                        timeRangesFeature={timeRangesHistogram}
                    />
                </div>
                <BryntumGrid
                    {...gridConfig}
                    tbar={[
                        {
                            text: 'Add order',
                            cls: {'tbarBryntumButton': true},
                            //icon: 'b-icon b-fa-plus',
                            toggleable: true,
                            // onAction   : onAddClick
                        },
                        '->',
                        {
                            text: 'Hide scheduled',
                            //toggleable: true,
                            cls: {'tbarBryntumButton': true},
                            onAction: ({source}: any) => {
                                //const store = source.up('grid').store
                                const store: Store = gridInstance().store as Store
                                if (!store.isFiltered) {
                                    store.addFilter({
                                        property: 'startDate',
                                        operator: '=',
                                        value: null
                                    })
                                } else {
                                    store.clearFilters()
                                }
                            }
                        }
                    ]}
                    ref={gridRef}
                    data={extractData(props)}
                />
            </Fragment>
        </div>
    )
}

export default Timeline
export {View, ViewLength}
export type {CalendarType, TimelineProps, IdCalendar}